-- Author	  : admin
-- Create Date : 9/16/2020 5:12:50 PM

local addonName, addon = ...;
local module = addon:NewModule('CraftUI', 'AceEvent-3.0');
local name = 'Enchant';
local levels = newArray('trivial', 'easy', 'medium', 'optimal', 'difficult');

module.professionName = 'UNKNOWN';
module.GetNumItems = 'GetNumCrafts';
module.GetItemInfo = 'GetCraftInfo';
module.GetItemLink = 'GetCraftItemLink';
module.GetItemNumReagents = 'GetCraftNumReagents';
module.GetItemReagentInfo = 'GetCraftReagentInfo';
module.ExpandItemHeader = 'ExpandCraftSkillLine';
module.CollapseItemHeader = 'CollapseCraftSkillLine';

local GetCraftSelectionIndex = _G.GetCraftSelectionIndex;

local genericHooks = {
	'CollapseCraftSkillLine',
	'DoCraft',
	'ExpandCraftSkillLine',
	'GetCraftDescription',
	'GetCraftIcon',
	'GetCraftInfo',
	'GetCraftItemLink',
	'GetCraftNumReagents',
	'GetCraftReagentInfo',
	'GetCraftReagentItemLink',
	'GetCraftSkillLine',
	'GetCraftSpellFocus'
};

--'GetCraftSelectionIndex()
--'GetCraftButtonToken()
-- 'GetCraftName()
--'GetCraftDisplaySkillLine()
--'SelectCraft(index)'
--'GetNumCrafts()'
--'CloseCraft

function module:OnInitialize()
	module:SetEnabledState(false);

    hooksecurefunc('FauxScrollFrame_Update', module.FauxScrollFrame_Update);
end

function module:OnEnable()
	addon:initFrames(CraftFrame, module);

   	module:RegisterEvent('CRAFT_UPDATE');
	CraftFrame:UnregisterEvent('CRAFT_UPDATE');

    for i=1, CRAFTS_DISPLAYED, 1 do
        _G['Craft' .. i]:SetAttribute('id', i);
    end

    hooksecurefunc('Craft_UpdateTrainingPoints', module.Craft_UpdateTrainingPoints);
end

--
-- methodes
--

function module:getItem(button)
    local index = FauxScrollFrame_GetOffset(CraftListScrollFrame) + button:GetAttribute('id');

    return addon:getSkillData(module.professionName).items[index];
end

local GetCraftInfo = _G.GetCraftInfo;
function module.GetCraftInfo(index)
	local data = {GetCraftInfo(index)};
	table.remove(data, 2, 1)

	return unpack(data);
end

function module:isSelectedCraftVisible()
    local items = addon:getSkillData(module.professionName).items;

    -- correct selection
    local selectedVisible = false
    for _, item in ipairs(items) do
        if (item.index == GetCraftSelectionIndex()) then
            return true
        end
    end

    return false;
end


function module:updateFrame()
	securecall(CraftFrame_OnEvent, CraftFrame, 'CRAFT_UPDATE');
end

--
-- Events
--

function module:CRAFT_UPDATE(...)
	if (addon.scanning ~= nil) then
		return;
	end

	addon.scanning = module;
	module.professionName = GetCraftName() or 'UNKNOWN';

	addon:initDB(module.professionName);
	addon:updateButtons();

	addon:scan()
	addon.scanning = nil;

	securecall(CraftFrame:GetScript('OnEvent'), CraftFrame, ...);
end


--
-- Hooks
--

function module.Craft_UpdateTrainingPoints()
    if (module.updating) then
        return;
    end
    
    module.updating = true;
    local numCrafts = module:GetNumCrafts()
    local craftOffset = FauxScrollFrame_GetOffset(CraftListScrollFrame);
    local items = addon:getSkillData(module.professionName).items;

    if (module:isSelectedCraftVisible() == false) then
        securecall(FauxScrollFrame_SetOffset, CraftListScrollFrame,  0);

        -- select first when possible
        securecall(CraftFrame_SetSelection, #items > 0 and items[1].index);
    end

    --
    -- code from BLIZZARD_CRAFTUI start
    --
    for i=1, CRAFTS_DISPLAYED, 1 do
        craftButton = getglobal("Craft"..i);
        -- @HACK: correct mocked data index
        craftIndex = craftButton.craftItem and craftButton.craftItem.index or (numCrafts + 1);
        craftName, craftSubSpellName, craftType, numAvailable, isExpanded, trainingPointCost, requiredLevel = GetCraftInfo(craftIndex);
        craftButtonSubText = getglobal("Craft"..i.."SubText");
        craftButtonCost = getglobal("Craft"..i.."Cost");
        craftButtonText = getglobal("Craft"..i.."Text");
        -- @Hack: calc real index, not craftIndex, which can be higher
        if ((i + craftOffset) <= numCrafts ) then	
            -- Set button widths if scrollbar is shown or hidden
            if ( CraftListScrollFrame:IsVisible() ) then
                craftButton:SetWidth(293);
                -- HACK to fix a bug with (Rank) rendering in Beast Training --
                if ( not trainingPointCost ) then
                    craftButtonText:SetWidth(290);
                end
            else
                craftButton:SetWidth(323);
                -- HACK to fix a bug with (Rank) rendering in Beast Training --
                if ( not trainingPointCost ) then
                    craftButtonText:SetWidth(320);
                end
            end
            craftButtonCost:SetPoint("RIGHT", CRAFT_COST_OFFSET, 0);
            local color = CraftTypeColor[craftType];

            craftButton:SetNormalFontObject(color.font);
            craftButtonCost:SetTextColor(color.r, color.g, color.b);
            Craft_SetSubTextColor(craftButton, color.r, color.g, color.b);
            craftButton:SetID(craftIndex);
            craftButton:Show();
            -- Handle headers
            if ( craftType == "header" ) then
                craftButton:SetText(craftName);
                craftButtonSubText:SetText("");
                craftButtonText:SetWidth(0);
                if ( isExpanded ) then
                    craftButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
                else
                    craftButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
                end
                getglobal("Craft"..i.."Highlight"):SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
                getglobal("Craft"..i):UnlockHighlight();
            else
                craftButton:SetNormalTexture("");
                getglobal("Craft"..i.."Highlight"):SetTexture("");
                if ( numAvailable == 0 ) then
                    craftButton:SetText(" "..craftName);
                else
                    craftButton:SetText(" "..craftName.." ["..numAvailable.."]");
                end
                if ( craftSubSpellName and craftSubSpellName ~= "" ) then
                    craftButtonSubText:SetText(format(PARENS_TEMPLATE, craftSubSpellName));
                    craftButtonText:SetWidth(0);
                else 
                    craftButtonSubText:SetText("");

                    -- A bit of a hack. If there's no subtext, we'll set a width to ensure that we don't overflow.
                    craftButtonText:SetWidth(CRAFT_TEXT_WIDTH);
                end
                if ( trainingPointCost > 0 ) then
                    craftButtonCost:SetText(format(TRAINER_LIST_TP, trainingPointCost));
                else
                    craftButtonCost:SetText("");
                end

                craftButtonSubText:SetPoint("LEFT", "Craft"..i.."Text", "RIGHT", CRAFT_SUBTEXT_OFFSET, 0);
                -- Place the highlight and lock the highlight state
                if ( GetCraftSelectionIndex() == craftIndex ) then
                    CraftHighlightFrame:SetPoint("TOPLEFT", "Craft"..i, "TOPLEFT", 0, 0);
                    CraftHighlightFrame:Show();
                    Craft_SetSubTextColor(craftButton, 1.0, 1.0, 1.0);
                    craftButtonCost:SetTextColor(1.0, 1.0, 1.0);
                    getglobal("Craft"..i):LockHighlight();
                else
                    getglobal("Craft"..i):UnlockHighlight();
                end
            end

        else
            craftButton:Hide();
        end
    end

    -- code from BLIZZARD_CRAFTUI end

    module.updating = nil;
end


local callFauxScrollFrame_UpdateHook = false;
function module.FauxScrollFrame_Update(frame, numLines, ...)
    if (frame ~= CraftListScrollFrame or callFauxScrollFrame_UpdateHook) then
        return
    end
    
    callFauxScrollFrame_UpdateHook = true;
    securecall(FauxScrollFrame_Update, frame, module:GetNumCrafts(), ...);
    callFauxScrollFrame_UpdateHook = false;

    for i=1, CRAFTS_DISPLAYED, 1 do
        local button = _G['Craft' .. i];

        button.craftItem = module:getItem(button);
    end
end

function module:GetNumCrafts()
    if (GetNumCrafts() == 0) then
        -- fix possibble call when craft is closed but an update was triggered by 'SKILL_LINES_CHANGED' event
        return 0;
    end

	return #addon:getSkillData(module.professionName).items
end
